package cd.casic.module.machine.controller;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.module.machine.service.MachineEnvService;
import cd.casic.module.machine.dto.MachineEnvDTO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import java.util.List;

import static cd.casic.framework.commons.pojo.CommonResult.success;


/**
 * 环境变量控制器
 */
@RestController
@RequestMapping("/api/machineEnv")
@Tag(name = "环境变量管理")
@RequiredArgsConstructor
public class MachineEnvController {

    @Resource
    private MachineEnvService machineEnvService;

    @PostMapping("/add")
    @Operation(summary = "新增环境变量")
    public CommonResult add(@RequestBody MachineEnvDTO machineEnvDTO) {
        machineEnvService.add(machineEnvDTO);
        return success(true);
    }

    @PutMapping("/update")
    @Operation(summary = "修改环境变量")
    public CommonResult update(@RequestBody MachineEnvDTO machineEnvDTO) {
        machineEnvService.update(machineEnvDTO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除机器的环境变量")
    public CommonResult deleteByMachineId(
            @RequestParam Long machineId) {
        machineEnvService.deleteByMachineId(machineId);
        return success(true);
    }

    @DeleteMapping("/deleteList")
    @Operation(summary = "批量删除机器环境变量")
    public CommonResult deleteList(@RequestParam List<Long> ids){
        machineEnvService.deleteList(ids);
        return success(true);
    }

    @GetMapping("/getByMachineId")
    @Operation(summary = "获取机器的环境变量")
    public CommonResult getByMachineId(
            @RequestParam Long machineId) {
        return success(machineEnvService.getByMachineId(machineId));
    }


    @PostMapping("/list")
    @Operation(summary ="获取环境变量列表")
    public CommonResult list(@RequestBody MachineEnvDTO machineEnvDTO) {
        return success(machineEnvService.listEnv(machineEnvDTO));
    }
}
