package cd.casic.module.machine.controller;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.module.machine.entity.MachineInfo;
import cd.casic.module.machine.service.MachineInfoService;
import cd.casic.module.machine.dto.MachineInfoDto;
import cd.casic.module.machine.pojo.SuccessResponseData;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.framework.commons.pojo.CommonResult.success;

@RestController
@Tag(name = "机器信息管理")
@RequestMapping("/api/machineInfo")
public class MachineInfoController {
    @Resource
    private MachineInfoService machineInfoService;

    @PostMapping("/add")
    @Operation(summary = "新增机器信息")
    public CommonResult add(@RequestBody MachineInfoDto machineInfoDto) {
        machineInfoService.addMachineInfo(machineInfoDto);
        return success(true);
    }


    @GetMapping("/list")
    @Operation(summary = "获取机器信息列表")
    public CommonResult list(@RequestBody MachineInfoDto machineInfoDto) {
        return success(machineInfoService.listMachineInfo(machineInfoDto));
    }

    @PutMapping("/update")
    @Operation(summary = "编辑机器信息")
    public CommonResult update(@RequestBody MachineInfoDto machineInfoDto) {
        machineInfoService.updateMachineInfo(machineInfoDto);
        return success(true);
    }

    @PutMapping("/updateStatus")
    @Operation(summary = "机器启用/停用")
    public CommonResult updateStatus(@RequestParam("machineInfoId") Long machineInfoId, @RequestParam("status") String status) {
       machineInfoService.updateStatus(machineInfoId, status);
        return success(true);
    }

    @PutMapping("/bindingSecretKey")
    @Operation(summary = "绑定密钥")
    public CommonResult bindingSecretKey(@RequestParam("machineInfoId") Long machineInfoId, @RequestParam("secretKeyId") Long secretKeyId) {
        machineInfoService.bindingSecretKey(machineInfoId, secretKeyId);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "机器信息删除")
    public CommonResult delete(@RequestParam("machineInfoId") Long machineInfoId) {
        machineInfoService.deleteMachineInfo(machineInfoId);
        return success(true);
    }

    @DeleteMapping("/deleteList")
    @Operation(summary = "批量删除机器信息")
    public CommonResult deleteList(@RequestParam("machineInfoId") List<Long> machineInfoIds) {
        machineInfoService.deleteList(machineInfoIds);
        return success(true);
    }

    @PostMapping("/test")
    @Operation(summary = "测试机器连接")
    public CommonResult testConnection(@RequestBody MachineInfo machineInfo) {
        machineInfoService.testConnection(machineInfo);
        return success(true);
    }

    @GetMapping("/status/{machineName}")
    @Operation(summary = "获取机器连接状态")
    public CommonResult getConnectionStatus(@PathVariable String machineName) {
        return success(machineInfoService.getConnectionStatus(machineName));
    }

    @GetMapping("/status/all")
    @Operation(summary = "获取所有机器连接状态")
    public CommonResult getAllConnectionStatus() {
        return success(machineInfoService.getAllConnectionStatus());
    }

    @PostMapping("/connect")
    @Operation(summary = "建立机器连接")
    public CommonResult connect(@RequestBody MachineInfo machineInfo) {
        return success(machineInfoService.connect(machineInfo));
    }

    @PostMapping("/disconnect/{sessionId}")
    @Operation(summary = "断开机器连接")
    public CommonResult disconnect(@PathVariable String sessionId) {
        machineInfoService.disconnect(sessionId);
        return success(true);
    }

    @PostMapping("/execute/{sessionId}")
    @Operation(summary = "执行远程命令")
    public CommonResult executeCommand(
            @PathVariable String sessionId,
            @RequestBody String command) {

        return success(machineInfoService.executeCommand(sessionId, command));
    }

    @PostMapping("/upload/{sessionId}")
    @Operation(summary = "上传文件到远程机器")
    public CommonResult uploadFile(
            @PathVariable String sessionId,
            @RequestParam String localFilePath,
            @RequestParam String remoteFilePath) {
        machineInfoService.uploadFile(sessionId, localFilePath, remoteFilePath);
        return success(true);
    }

    @PostMapping("/download/{sessionId}")
    @Operation(summary = "从远程机器下载文件")
    public CommonResult downloadFile(
            @PathVariable String sessionId,
            @RequestParam String remoteFilePath,
            @RequestParam String localFilePath) {
        machineInfoService.downloadFile(sessionId, remoteFilePath, localFilePath);
        return success(true);
    }
}
