package cd.casic.module.machine.controller;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.module.machine.service.MachineProxyService;
import cd.casic.module.machine.dto.MachineProxyDTO;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.framework.commons.pojo.CommonResult.success;

/**
 * 机器代理控制器
 */
@RestController
@RequestMapping("/api/machineProxy")
@Tag(name = "机器代理管理")
@RequiredArgsConstructor
public class MachineProxyController {

    @Resource
    private MachineProxyService machineProxyService;

    @PostMapping("/register")
    @Operation(summary ="注册新的机器代理")
    public CommonResult register(@RequestBody MachineProxyDTO machineProxyDTO) {
        machineProxyService.register(machineProxyDTO);
        return success(true);
    }

    @GetMapping("/list")
    @Operation(summary ="获取代理列表")
    public CommonResult list(MachineProxyDTO machineProxyDTO) {
        return success(machineProxyService.list(machineProxyDTO));
    }

    @PutMapping("/updateStatus")
    @Operation(summary ="更新代理状态")
    public CommonResult updateStatus(@RequestBody MachineProxyDTO machineProxyDTO) {
        machineProxyService.updateStatus(machineProxyDTO);
        return success(true);
    }

    @GetMapping("/statistics/status")
    @Operation(summary ="获取所有代理的状态统计")
    public CommonResult getStatusStatistics() {
        return success(machineProxyService.getStatusStatistics());
    }

    @PutMapping("/updateConfig")
    @Operation(summary ="更新代理配置")
    public CommonResult updateConfig(@RequestBody MachineProxyDTO machineProxyDTO) {
        machineProxyService.updateConfig(machineProxyDTO);
        return success(true);
    }

    @DeleteMapping("/batch")
    @Operation(summary ="批量删除代理")
    public CommonResult deleteBatch(@RequestParam List<Long> ids) {
        machineProxyService.delete(ids);
        return success(true);
    }

}
