package cd.casic.module.machine.dto;
import cd.casic.module.machine.enums.MachineProxyStatus;
import lombok.*;

import java.io.Serializable;
import java.util.Date;
import java.util.Objects;

/**
 * 机器代理数据传输对象
 */
@EqualsAndHashCode(callSuper = true)
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class MachineProxyDTO extends PageDto implements Serializable {
    private static final long serialVersionUID = 1L;

    private Long id;
    private String username;
    private String proxyType;
    private String version;
    private String status;
    private Date lastHeartbeatTime;
    private String config;
    private String description;
    private String hostIp;
    private String sshPort;
    private Date createDate;
    private Date updateDate;

    /**
     * 计算代理是否在线
     */
    public boolean isOnline() {
        if (status == null || lastHeartbeatTime == null) {
            return false;
        }
        // 假设5分钟内有心跳为在线
        long fiveMinutes = 5 * 60 * 1000;
        return Objects.equals(MachineProxyStatus.ONLINE.getMessage(), status) &&
                System.currentTimeMillis() - lastHeartbeatTime.getTime() < fiveMinutes;
    }
}
