package cd.casic.module.machine.service.impl;

import cd.casic.module.machine.dto.SecretKeyDto;
import cd.casic.module.machine.entity.MachineInfo;
import cd.casic.module.machine.entity.SecretKey;
import cd.casic.module.machine.mapper.SecretServiceMapper;
import cd.casic.module.machine.utils.AliOssUtil;
import cd.casic.module.machine.utils.PageResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cd.casic.module.machine.configuration.AliYunConfig;
import cd.casic.module.machine.exception.ServiceException;
import cd.casic.module.machine.service.MachineInfoService;
import cd.casic.module.machine.service.SecretKeyService;
import jakarta.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Service
public class SecretKeyServiceImpl extends ServiceImpl<SecretServiceMapper, SecretKey> implements SecretKeyService {
    @Resource
    private AliOssUtil aliOssUtil;

    @Resource
    private MachineInfoService machineInfoService;

//    @Resource
    private AliYunConfig aliYunConfig;

    @Resource
    private SecretServiceMapper secretServiceMapper;



    //todo public方便测试，后面改为private
    public static final ExecutorService FILE_DELETE_EXECUTOR = Executors.newFixedThreadPool(10);

    Logger logger = LoggerFactory.getLogger(SecretKeyServiceImpl.class);

    @Override
    public boolean addSecretKey(SecretKeyDto secretKeyDto, MultipartFile file) {
        secretKeyDto.setFile(file);
        String fileName = aliOssUtil.save(file);
        String path = "https://" + aliYunConfig.getBucketName() + "." + aliYunConfig.getEndpoint() + "/" + fileName;
        secretKeyDto.setPath(path);
        secretKeyDto.setFileName(fileName);
        SecretKey secretKey = new SecretKey();
        BeanUtils.copyProperties(secretKeyDto,secretKey);
        return this.save(secretKey);
    }

    @Override
    public void bindingMachine(Long secretKeyId, List<Long> machineInfoIds) {
        List<MachineInfo> machineInfos = machineInfoService.listByIds(machineInfoIds);
        machineInfos.forEach(machineInfo -> machineInfo.setSecretKeyId(secretKeyId));
    }

    @Override
    public boolean updateSecretKey(SecretKeyDto secretKeyDto, MultipartFile file) {
        if (file != null){
            secretKeyDto.setFile(file);
            String fileName = aliOssUtil.save(file);
            String path = "https://" + aliYunConfig.getBucketName() + "." + aliYunConfig.getEndpoint() + "/" + fileName;
            secretKeyDto.setPath(path);
            secretKeyDto.setFileName(fileName);
        }
        SecretKey secretKey = new SecretKey();
        BeanUtils.copyProperties(secretKeyDto,secretKey);
        return this.updateById(secretKey);
    }

    @Override
    public boolean deleteSecretKey(Long secretKeyId) {
        String fileName = this.getById(secretKeyId).getFileName();
        aliOssUtil.deleteFile(fileName);
        return this.removeById(secretKeyId);
    }

    @Override
    public PageResult<SecretKey> listSecretKey(SecretKeyDto secretKeyDto) {
        QueryWrapper<SecretKey> queryWrapper = new QueryWrapper<>();
        if (secretKeyDto.getName() != null && !secretKeyDto.getName().isEmpty()){
            queryWrapper.like("name", secretKeyDto.getName());
        }
        if (secretKeyDto.getDescription() != null && !secretKeyDto.getDescription().isEmpty()){
            queryWrapper.like("description", secretKeyDto.getDescription());
        }
        Page<SecretKey> page = secretServiceMapper.selectPage(new Page<>(secretKeyDto.getPageIndex(), secretKeyDto.getPageSize()), queryWrapper);
        return new  PageResult<>(
            page.getCurrent(),
            page.getSize(),
            page.getTotal(),
            page.getPages(),
            page.getRecords()
        );
    }

    @Override
    public ResponseEntity<InputStreamResource> downloadSecretKeyFile(Long secretKeyId) {
        String fileName = this.getById(secretKeyId).getFileName();
        return aliOssUtil.downloadFile(fileName);
    }

    @Override
    public boolean deleteList(List<Long> secretKeyIds) {
        List<SecretKey> secretKeys = this.listByIds(secretKeyIds);
        // 提交异步任务到线程池
        FILE_DELETE_EXECUTOR.execute(() -> {
            try {
                for (SecretKey secretKey : secretKeys) {
                    if (secretKey.getFileName() != null && !secretKey.getFileName().isEmpty()){
                        aliOssUtil.deleteFile(secretKey.getFileName());
                    }
                }
            } catch (Exception e) {
                logger.error("异步删除文件失败：{}", e.getMessage());
                throw new ServiceException(ServiceException.DELETE_FILE_FAIL,"异步删除文件失败:"+e.getMessage());
            }
        });
        return secretServiceMapper.deleteBatchIds(secretKeyIds) > 0 ;
    }
}
