package cd.casic.module.process.support.postprocess.service.impl;

import cd.casic.ci.commons.bean.engine.job.Tasks;
import cd.casic.ci.commons.bean.engine.postprocess.Postprocess;
import cd.casic.ci.commons.bean.exception.ApplicationException;
import cd.casic.module.process.engine.task.TasksCloneService;
import cd.casic.module.process.engine.task.TasksService;
import cd.casic.module.process.support.postprocess.dao.PostprocessDao;
import cd.casic.module.process.support.postprocess.entity.PostprocessDo;
import cd.casic.module.process.support.postprocess.service.PostprocessService;
import cd.casic.module.process.toolkit.beans.BeanMapper;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;

/**
 * @ClassName PostprocessServiceImpl
 * @Author hopeli
 * @Date 2025/4/28 16:26
 * @Version 1.0
 */
@Service
public class PostprocessServiceImpl implements PostprocessService {

    @Resource
    PostprocessDao postprocessDao;

    @Resource
    TasksService tasksService;

    @Resource
    TasksCloneService tasksCloneService;

    @Override
    public String createPostTask(Postprocess postprocess) {
        if (Objects.isNull(postprocess.getTaskType())){
            throw new ApplicationException("taskType不能为空！");
        }

        String pipelineId = postprocess.getPipelineId();
        if (Objects.isNull(pipelineId)){
            List<PostprocessDo> taskPost = postprocessDao.findTaskPost(postprocess.getTaskId());
            int size = taskPost.size();
            postprocess.setTaskSort(size);
        }else {
            List<PostprocessDo> taskPost = postprocessDao.findPipelinePost(pipelineId);
            int size = taskPost.size();
            postprocess.setTaskSort(size);
        }

        // 设置默认名称
        if (Objects.isNull(postprocess.getPostName())){
            String name = tasksService.initDifferentTaskName(postprocess.getTaskType());
            postprocess.setPostName(name);
        }

        PostprocessDo postprocessEntity = BeanMapper.map(postprocess, PostprocessDo.class);
        postprocessEntity.setCreateTime(LocalDateTime.now());
        String postId = postprocessDao.createPost(postprocessEntity);
        Tasks tasks = new Tasks();
        tasks.setTaskSort(postprocess.getTaskSort());
        tasks.setTaskName(postprocess.getPostName());
        tasks.setTaskType(postprocess.getTaskType());
        tasks.setPostprocessId(postId);
        tasks.setValues(postprocess.getValues());
        tasksService.createTasksOrTask(tasks);
        return postId;
    }

    @Override
    public List<Postprocess> findAllPipelinePostTask(String pipelineId){
        List<PostprocessDo> allPostEntity = postprocessDao.findPipelinePost(pipelineId);
        List<Postprocess> postprocessList = BeanMapper.mapList(allPostEntity, Postprocess.class);
        if (postprocessList.isEmpty()){
            return Collections.emptyList();
        }
        List<Postprocess> list = new ArrayList<>();
        for (Postprocess postprocess : postprocessList) {
            String postprocessId = postprocess.getPostId();
            Tasks tasks = tasksService.findOnePostTaskOrTask(postprocessId);
            postprocess.setTask(tasks);
            String taskType = tasks.getTaskType();
            postprocess.setTaskType(taskType);
            list.add(postprocess);
        }
        list.sort(Comparator.comparing(Postprocess::getTaskSort));
        return list;
    }

    @Override
    public void clonePostTask(String pipelineId,String clonePipelineId){
        List<PostprocessDo> allPostEntity = postprocessDao.findPipelinePost(pipelineId);
        List<Postprocess> postprocessList = BeanMapper.mapList(allPostEntity, Postprocess.class);
        for (Postprocess postprocess : postprocessList) {
            postprocess.setPipelineId(clonePipelineId);
            PostprocessDo postprocessEntity = BeanMapper.map(postprocess, PostprocessDo.class);
            String clonePostId = postprocessDao.createPost(postprocessEntity);
            tasksCloneService.clonePostTasks(postprocess.getPostId(),clonePostId);
        }
    }

    @Override
    public  List<Postprocess> findAllTaskPostTask(String taskId) {
        List<PostprocessDo> allPostEntity = postprocessDao.findTaskPost(taskId);
        List<Postprocess> postprocessList = BeanMapper.mapList(allPostEntity, Postprocess.class);
        if (postprocessList.isEmpty()){
            return Collections.emptyList();
        }
        List<Postprocess> list = new ArrayList<>();
        for (Postprocess postprocess : postprocessList) {
            String id = postprocess.getPostId();
            Tasks tasks = tasksService.findOnePostTaskOrTask(id);
            String taskType = tasks.getTaskType();
            postprocess.setTaskType(taskType);
            postprocess.setTask(tasks);
            list.add(postprocess);
        }
        postprocessList.sort(Comparator.comparing(Postprocess::getCreateTime).reversed());
        return list;
    }

    @Override
    public void deletePostTask(String postprocessId) {
        Tasks postTask = tasksService.findOnePostTask(postprocessId);
        String taskId = postTask.getTaskId();
        tasksService.deleteTasksOrTask(taskId);
        postprocessDao.deletePost(postprocessId);
    }

    @Override
    public List<Postprocess> findAllPost(String taskId) {
        List<Postprocess> allPostprocess = findAllPost();
        if (allPostprocess == null){
            return Collections.emptyList();
        }
        List<Postprocess> list = new ArrayList<>();
        for (Postprocess postprocess : allPostprocess) {
            String postTaskId = postprocess.getTaskId();
            if (!postTaskId.equals(taskId)){
                continue;
            }
            list.add(postprocess);
        }
        list.sort(Comparator.comparing(Postprocess::getCreateTime).reversed());
        return list;
    }

    @Override
    public void updatePostTask(Postprocess postprocess) {
        String postprocessId = postprocess.getPostId();
        Postprocess onePost = findOnePost(postprocessId);
        onePost.setPostName(postprocess.getPostName());
        postprocessDao.updatePost(BeanMapper.map(onePost,PostprocessDo.class));

        Tasks task = tasksService.findOnePostTask(postprocessId);
        Object values = postprocess.getValues();
        task.setTask(values);
        task.setValues(values);
        task.setTaskType(postprocess.getTaskType());
        tasksService.updateTasksTask(task);
    }

    public void updatePost(Postprocess postprocess){
        postprocessDao.updatePost(BeanMapper.map(postprocess,PostprocessDo.class));
    }

    @Override
    public Postprocess findOnePostOrTask(String postprocessId) {
        Postprocess postprocess = findOnePost(postprocessId);
        String id = postprocess.getPostId();
        Tasks taskOrTask = tasksService.findOnePostTaskOrTask(id);
        postprocess.setTask(taskOrTask);
        postprocess.setTaskType(taskOrTask.getTaskType());
        return postprocess;
    }

    @Override
    public Postprocess findOnePost(String postprocessId) {
        PostprocessDo postprocessEntity = postprocessDao.findOnePost(postprocessId);
        return BeanMapper.map(postprocessEntity, Postprocess.class);
    }

    @Override
    public List<Postprocess> findAllPost() {
        return BeanMapper.mapList(postprocessDao.findAllPost(), Postprocess.class);
    }

    @Override
    public List<Postprocess> findAllPostList(List<String> idList) {
        return BeanMapper.mapList(postprocessDao.findAllPostList(idList), Postprocess.class);
    }
}
