package cd.casic.module.process.toolkit.beans.metadata;

import cd.casic.ci.commons.bean.exception.SystemException;
import cd.casic.module.process.core.resolver.AnnotationResourceResolver;
import cd.casic.module.process.toolkit.beans.annotation.Mapper;
import cd.casic.module.process.toolkit.beans.annotation.Mappers;
import cd.casic.module.process.toolkit.beans.annotation.Mapping;
import cd.casic.module.process.toolkit.beans.annotation.Mappings;
import cd.casic.module.process.toolkit.beans.model.BeanMapping;
import cd.casic.module.process.toolkit.beans.model.FieldMapping;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.util.*;

public class BeanMapperRegister {
    private static Map<String, BeanMapping> mappers = new HashMap();
    private static final String RELA_SPLIT = "->";

    public BeanMapperRegister() {
    }

    public void scan(String basePackage) {
        Set<Class> classSet = AnnotationResourceResolver.resolve(basePackage, Mapper.class);
        Iterator var3;
        Class cls;
        if (classSet != null && classSet.size() > 0) {
            var3 = classSet.iterator();

            while(var3.hasNext()) {
                cls = (Class)var3.next();
                Mapper mapper = (Mapper)cls.getAnnotation(Mapper.class);
                this.parse(cls, mapper);
            }
        }

        classSet = AnnotationResourceResolver.resolve(basePackage, Mappers.class);
        if (classSet != null && classSet.size() > 0) {
            var3 = classSet.iterator();

            while(var3.hasNext()) {
                cls = (Class)var3.next();
                Mappers mappers = (Mappers)cls.getAnnotation(Mappers.class);
                Mapper[] var6 = mappers.value();
                int var7 = var6.length;

                for(int var8 = 0; var8 < var7; ++var8) {
                    Mapper mapper = var6[var8];
                    this.parse(cls, mapper);
                }
            }
        }

    }

    public BeanMapperRegister map(Class sourceClass, Class targetClass) {
        BeanMapping mapperMeta = new BeanMapping();
        mapperMeta.setSourceClass(sourceClass);
        mapperMeta.setTargetClass(targetClass);
        return this;
    }

    void parse(Class mapperClz, Mapper mapper) {
        BeanMapping beanMapping = null;

        try {
            beanMapping = this.parseBeanMapping(mapperClz, mapper);
        } catch (Exception var7) {
            return;
        }

        mappers.put(getBeanMappingKey(beanMapping), beanMapping);
        BeanMapping reverseBeanMapping = null;

        try {
            reverseBeanMapping = this.parseBeanMappingForReverse(beanMapping);
        } catch (Exception var6) {
            return;
        }

        mappers.put(getBeanMappingKey(reverseBeanMapping), reverseBeanMapping);
    }

    public BeanMapping parseBeanMappingForReverse(BeanMapping classMapping) {
        BeanMapping reverseBeanMapping = new BeanMapping();
        reverseBeanMapping.setSourceClass(classMapping.getTargetClass());
        reverseBeanMapping.setTargetClass(classMapping.getSourceClass());
        Iterator var3 = classMapping.getPropertyMappingList().iterator();

        while(var3.hasNext()) {
            FieldMapping item = (FieldMapping)var3.next();
            FieldMapping propertyMapping = new FieldMapping();
            propertyMapping.setSource(item.getTarget());
            propertyMapping.setTarget(item.getSource());
            reverseBeanMapping.getPropertyMappingList().add(propertyMapping);
        }

        return reverseBeanMapping;
    }

    BeanMapping parseBeanMapping(Class mapperClz, Mapper mapper) {
        try {
            BeanMapping beanMapping = new BeanMapping();
            Class sourceClass = mapper.source();
            if (sourceClass == Object.class) {
                sourceClass = mapperClz;
            }

            Class targetClass = mapper.target();
            if (targetClass == Object.class) {
                String targetName = mapper.targetName();
                if (!StringUtils.isEmpty(targetName)) {
                    targetClass = this.getClassByName(targetName);
                } else {
                    targetClass = this.findTargetClassByRule(sourceClass);
                }
            }

            beanMapping.setSourceClass(sourceClass);
            beanMapping.setTargetClass(targetClass);
            List<FieldMapping> propertyMappingList = this.parsePropertiesMapping(beanMapping);
            beanMapping.setPropertyMappingList(propertyMappingList);
            return beanMapping;
        } catch (Exception var7) {
            throw new SystemException(String.format("parse class:%s failed.", mapperClz.getName()));
//            throw new SystemException(String.format("parse class:%s failed.", mapperClz.getName()), var7);
        }
    }

    public Class findTargetClassByRule(Class sourceClass) {
        String className = sourceClass.getName();
        String[] arr = className.split("\\.");
        StringBuffer buffer = new StringBuffer();

        for(int i = 0; i < arr.length; ++i) {
            String item = arr[i];
            if (i < arr.length - 2) {
                buffer.append(item).append(".");
            } else if (i == arr.length - 2) {
                buffer.append("entity").append(".");
            } else if (i == arr.length - 1) {
                String entityName = item + "Entity";
                buffer.append(entityName);
            }
        }

        String targetName = buffer.toString();
        Class targetClass = this.getClassByName(targetName);
        return targetClass;
    }

    Class getClassByName(String className) {
        try {
            Class targetClass = Class.forName(className);
            return targetClass;
        } catch (ClassNotFoundException var6) {
            try {
                ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                Class<?> clazz = classLoader.loadClass(className);
                return clazz;
            } catch (ClassNotFoundException var5) {
                throw new SystemException("class not found:" + className);
//                throw new SystemException("class not found:" + className, var6);
            }
        }
    }

    List<FieldMapping> parsePropertiesMapping(BeanMapping classMapping) {
        List<FieldMapping> propertyMappingList = new ArrayList();
        Class sourceClz = classMapping.getSourceClass();
        Class targetClz = classMapping.getTargetClass();
        Field[] fields = sourceClz.getDeclaredFields();
        if (fields != null && fields.length > 0) {
        }

        Field[] var6 = fields;
        int var7 = fields.length;

        for(int var8 = 0; var8 < var7; ++var8) {
            Field field = var6[var8];
            if (field.getAnnotation(Mapping.class) != null) {
                Mapping mapping = (Mapping)field.getAnnotation(Mapping.class);
                this.validField(sourceClz, mapping.source());
                this.validField(targetClz, mapping.target());
                FieldMapping propertyMapping = buildPropertyMapping(field, mapping);
                propertyMappingList.add(propertyMapping);
            } else if (field.getAnnotation(Mappings.class) != null) {
                Mappings mappings = (Mappings)field.getAnnotation(Mappings.class);
                Mapping[] var11 = mappings.value();
                int var12 = var11.length;

                for(int var13 = 0; var13 < var12; ++var13) {
                    Mapping mapping = var11[var13];
                    this.validField(sourceClz, mapping.source());
                    this.validField(targetClz, mapping.target());
                    FieldMapping propertyMapping = buildPropertyMapping(field, mapping);
                    propertyMappingList.add(propertyMapping);
                }
            } else {
                try {
                    this.validField(targetClz, field.getName());
                } catch (Exception var16) {
                    continue;
                }

                FieldMapping propertyMapping = new FieldMapping(field.getName(), field.getName());
                propertyMappingList.add(propertyMapping);
            }
        }

        return propertyMappingList;
    }

    void validField(Class clz, String fieldName) {
        if (!fieldName.contains(".")) {
            try {
                clz.getDeclaredField(fieldName);
            } catch (NoSuchFieldException var11) {
                throw new SystemException(String.format("mapping field not found,clz:%s,field:%s", clz.getName(), fieldName));
            }
        } else {
            String[] arr = fieldName.split("\\.");
            String parentFieldName = arr[0];
            Field parentField = null;

            try {
                parentField = clz.getDeclaredField(parentFieldName);
            } catch (NoSuchFieldException var10) {
                throw new SystemException(String.format("mapping field not found,clz:%s,field:%s", clz.getName(), parentFieldName));
            }

            String childFieldName = arr[1];
            Class parentFieldClz = null;

            try {
                parentFieldClz = parentField.getType();
                parentFieldClz.getDeclaredField(childFieldName);
            } catch (NoSuchFieldException var9) {
                throw new SystemException(String.format("mapping field not found,clz:%s,field:%s", parentFieldClz.getName(), childFieldName));
            }
        }

    }

    static FieldMapping buildPropertyMapping(Field field, Mapping mapping) {
        FieldMapping propertyMappingMeta = new FieldMapping();
        propertyMappingMeta.setSource(mapping.source());
        propertyMappingMeta.setTarget(mapping.target());
        return propertyMappingMeta;
    }

    public static BeanMapping getBeanMapping(Class sourceClass, Class targetClass) {
        String mappingKey = getBeanMappingKey(sourceClass, targetClass);
        BeanMapping classMapping = (BeanMapping)mappers.get(mappingKey);
        return classMapping;
    }

    static String getBeanMappingKey(BeanMapping beanMapping) {
        String var10000 = beanMapping.getSourceClass().getName();
        return var10000 + "->" + beanMapping.getTargetClass().getName();
    }

    static String getBeanMappingKey(Class sourceClass, Class targetClass) {
        String var10000 = sourceClass.getName();
        return var10000 + "->" + targetClass.getName();
    }
}
